#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include <glm/glm.hpp>
#include <vector>
#include "moglclass.h"
#include "shaders.h"
#include "vertexBufferObject.h"

using namespace moglclass;

#ifndef M_PI
#define M_PI 3.141592
#endif
#define MOGL_STEP 7
#define MOGL_THICKNESS 3

#define NUMSHADERS 2
Shader shShaders[NUMSHADERS];
Program spMain;

std::vector<glm::vec4> circleVertices;
GLuint vao;
GLuint vbo;
GLuint program;

int g_colormode = 0;
GLenum g_primitivemode = GL_POINTS;
bool g_wiremode = false;

////////////////////////////////////////////////////////////////////////////////////////////////////

void SetCircleData(float radius, int step)
{
	// circle position
	float theta = (float) (2*M_PI/step);
	for (int i=0; i<step; i++) 
	{
		float x = radius * cos(theta * i);
		float y = radius * sin(theta * i);
		circleVertices.push_back(glm::vec4(x, y, 0.0f, 1.0f));
	}
	printf("circle::numVertices =%d\n", (int)circleVertices.size());

	// VAO
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	// VBO
	glGenBuffers(1, &vbo);
	glBindBuffer(GL_ARRAY_BUFFER, vbo);
	glBufferData(GL_ARRAY_BUFFER, step*sizeof(glm::vec4), &circleVertices[0], GL_STATIC_DRAW);
	glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(0);
		
	glBindVertexArray(0);

	circleVertices.clear();
}

void init( void )
{
    // Load shaders and use the resulting shader program
	spMain.createProgram();
	shShaders[0].loadShader("simple2.vs", GL_VERTEX_SHADER);
	shShaders[1].loadShader("simple2.fs", GL_FRAGMENT_SHADER);
	spMain.attachShaderToProgram(&shShaders[0]);
	spMain.attachShaderToProgram(&shShaders[1]);
	spMain.linkProgram();
	spMain.useProgram();
	
	// Set data
	SetCircleData(0.75f, MOGL_STEP);

	glClearColor( 1.0, 1.0, 1.0, 1.0 ); // white background
}

void display( void )
{
    glClear( GL_COLOR_BUFFER_BIT );     // clear the window

	// circle
	glBindVertexArray(vao);
	if (g_colormode == 0) 
		glVertexAttrib3f((GLuint)1, 1, 0, 0); // set constant color attribute (red)
	else if (g_colormode == 1) 
		glVertexAttrib3f((GLuint)1, 0, 1, 0); // set constant color attribute (green)
	else if (g_colormode == 2) 
		glVertexAttrib3f((GLuint)1, 0, 0, 1); // set constant color attribute (blue)

	// set drawmode
	if (g_wiremode)
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);  // draw wireframe 
	else
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);	// draw filled 

	// primitive type
	if (g_primitivemode == GL_POINTS) // 0
	{
		glPointSize(MOGL_THICKNESS);
		glDrawArrays(GL_POINTS, 0, MOGL_STEP);       // draw points
		glPointSize(1);
	}
	else if (g_primitivemode == GL_LINES) // 1
	{
		glLineWidth(MOGL_THICKNESS);
		glDrawArrays(GL_LINES, 0, MOGL_STEP);       // draw lines
		glLineWidth(1);
	}
	else if (g_primitivemode == GL_LINE_LOOP) // 2
	{
		glLineWidth(MOGL_THICKNESS);
		glDrawArrays(GL_LINE_LOOP, 0, MOGL_STEP);       // draw line loop
		glLineWidth(1);
	}
	else if (g_primitivemode == GL_LINE_STRIP) // 3
	{
		glLineWidth(MOGL_THICKNESS);
		glDrawArrays(GL_LINE_STRIP, 0, MOGL_STEP);       // draw line strip
		glLineWidth(1);
	}
	else if (g_primitivemode == GL_TRIANGLES) // 4
	{
		glDrawArrays(GL_TRIANGLES, 0, MOGL_STEP);       // draw triangles
	}
	else if (g_primitivemode == GL_TRIANGLE_STRIP) // 5
	{
		glDrawArrays(GL_TRIANGLE_STRIP, 0, MOGL_STEP);       // draw triangle strip
	}
	else if (g_primitivemode == GL_TRIANGLE_FAN) // 6
	{
		glDrawArrays(GL_TRIANGLE_FAN, 0, MOGL_STEP);       // draw triangle fan
	}
	else if (g_primitivemode == GL_POLYGON) // 9
	{
		glDrawArrays(GL_POLYGON, 0, MOGL_STEP);       // draw polygon
	}

	glBindVertexArray(0);
	glDisableVertexAttribArray(0);

	glFlush();
}

void keyboard( unsigned char key, int x, int y )
{
	switch (key)  
	{    
	case 0x1B:    
	case 'q':    
	case 'Q':      
		exit(0);     
	case ' ':
		g_primitivemode++;
		if (g_primitivemode >= GL_POLYGON) g_primitivemode = GL_POINTS;
		else if (g_primitivemode > GL_TRIANGLE_FAN) g_primitivemode = GL_POLYGON;
		else g_primitivemode = g_primitivemode % 7;
		printf("g_primitivemode=%d\n", g_primitivemode);
		break;
	case 'c':
		g_colormode = (g_colormode+1) % 3;
		printf("g_colormode=%d\n", g_colormode);
		break;  
	case 'w':
		g_wiremode = !g_wiremode;
		break;
	}
	glutPostRedisplay();
}

void reshape(int w, int h)
{
	glViewport(0, 0, w, h);
}

void cleanup()
{
	spMain.deleteProgram();
}

int main( int argc, char **argv )
{
	glutInit(&argc, argv);
    glutInitDisplayMode( GLUT_RGBA );
    glutInitWindowSize( 512, 512 );
    glutCreateWindow(argv[0]);
    
	glewInit();
    init();
    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
	glutReshapeFunc( reshape );
	glutMainLoop();

	cleanup();
    return 0;
}
